@@ -37,7 +37,7 @@ bool swoole_websocket_handshake(swoole::http::Context *ctx);
 
 #ifdef SW_USE_HTTP2
 
-int swoole_http2_server_onFrame(swoole::Server *serv, swoole::Connection *conn, swoole::RecvData *req);
+int swoole_http2_server_onReceive(swoole::Server *serv, swoole::Connection *conn, swoole::RecvData *req);
 int swoole_http2_server_parse(swoole::http2::Session *client, const char *buf);
 void swoole_http2_server_session_free(swoole::Connection *conn);
 int swoole_http2_server_ping(swoole::http::Context *ctx);
