@@ -19,26 +19,29 @@
 #include <string>
 #include <map>
 
+using swoole::microtime;
 using swoole::PHPCoroutine;
+using swoole::Server;
+using swoole::String;
 using swoole::coroutine::Socket;
 using swoole::coroutine::System;
-using swoole::microtime;
 
-using http_request = swoole::http::Request;
-using http_response = swoole::http::Response;
-using http_context = swoole::http::Context;
+using HttpRequest = swoole::http::Request;
+using HttpResponse = swoole::http::Response;
+using HttpContext = swoole::http::Context;
 
 #ifdef SW_USE_HTTP2
-using Http2Stream = swoole::http2::Stream;
-using Http2Session = swoole::http2::Session;
+namespace http2 = swoole::http2;
+using Http2Stream = http2::Stream;
+using Http2Session = http2::Session;
 #endif
 
 static zend_class_entry *swoole_http_server_coro_ce;
 static zend_object_handlers swoole_http_server_coro_handlers;
 
-static bool http_context_send_data(http_context *ctx, const char *data, size_t length);
-static bool http_context_sendfile(http_context *ctx, const char *file, uint32_t l_file, off_t offset, size_t length);
-static bool http_context_disconnect(http_context *ctx);
+static bool http_context_send_data(HttpContext *ctx, const char *data, size_t length);
+static bool http_context_sendfile(HttpContext *ctx, const char *file, uint32_t l_file, off_t offset, size_t length);
+static bool http_context_disconnect(HttpContext *ctx);
 
 #ifdef SW_USE_HTTP2
 static void http2_server_onRequest(Http2Session *session, Http2Stream *stream);
@@ -69,7 +72,7 @@ class http_server {
     uint32_t compression_min_length;
 #endif
 
-    http_server(enum swSocket_type type) {
+    http_server(enum swSocketType type) {
         socket = new Socket(type);
         default_handler = nullptr;
         array_init(&zcallbacks);
@@ -102,7 +105,7 @@ class http_server {
         add_assoc_zval_ex(&zcallbacks, pattern.c_str(), pattern.length(), zcallback);
     }
 
-    zend_fcall_info_cache *get_handler(http_context *ctx) {
+    zend_fcall_info_cache *get_handler(HttpContext *ctx) {
         for (auto i = handlers.begin(); i != handlers.end(); i++) {
             if (&i->second == default_handler) {
                 continue;
@@ -114,8 +117,8 @@ class http_server {
         return default_handler;
     }
 
-    http_context *create_context(Socket *conn, zval *zconn) {
-        http_context *ctx = swoole_http_context_new(conn->get_fd());
+    HttpContext *create_context(Socket *conn, zval *zconn) {
+        HttpContext *ctx = swoole_http_context_new(conn->get_fd());
         ctx->parse_body = http_parse_post;
         ctx->parse_cookie = http_parse_cookie;
         ctx->parse_files = http_parse_files;
@@ -141,15 +144,15 @@ class http_server {
     }
 
 #ifdef SW_USE_HTTP2
-    void recv_http2_frame(http_context *ctx) {
+    void recv_http2_frame(HttpContext *ctx) {
         Socket *sock = (Socket *) ctx->private_data;
-        swHttp2_send_setting_frame(&sock->protocol, sock->get_socket());
+        http2::send_setting_frame(&sock->protocol, sock->get_socket());
 
         sock->open_length_check = true;
         sock->protocol.package_length_size = SW_HTTP2_FRAME_HEADER_SIZE;
         sock->protocol.package_length_offset = 0;
         sock->protocol.package_body_offset = 0;
-        sock->protocol.get_package_length = swHttp2_get_frame_length;
+        sock->protocol.get_package_length = http2::get_frame_length;
 
         Http2Session session(ctx->fd);
         session.default_ctx = ctx;
@@ -247,17 +250,17 @@ static inline void http_server_set_error(zval *zobject, Socket *sock) {
     zend_update_property_string(swoole_http_server_coro_ce, SW_Z8_OBJ_P(zobject), ZEND_STRL("errMsg"), sock->errMsg);
 }
 
-static bool http_context_send_data(http_context *ctx, const char *data, size_t length) {
+static bool http_context_send_data(HttpContext *ctx, const char *data, size_t length) {
     Socket *sock = (Socket *) ctx->private_data;
     return sock->send_all(data, length) == (ssize_t) length;
 }
 
-static bool http_context_sendfile(http_context *ctx, const char *file, uint32_t l_file, off_t offset, size_t length) {
+static bool http_context_sendfile(HttpContext *ctx, const char *file, uint32_t l_file, off_t offset, size_t length) {
     Socket *sock = (Socket *) ctx->private_data;
     return sock->sendfile(file, offset, length);
 }
 
-static bool http_context_disconnect(http_context *ctx) {
+static bool http_context_disconnect(HttpContext *ctx) {
     Socket *sock = (Socket *) ctx->private_data;
     return sock->close();
 }
@@ -272,7 +275,7 @@ static void php_swoole_http_server_coro_free_object(zend_object *object) {
     zend_object_std_dtor(&hsc->std);
 }
 
-void http_context::init(Socket *sock) {
+void HttpContext::init(Socket *sock) {
     parse_cookie = 1;
     parse_body = 1;
     parse_files = 1;
@@ -287,7 +290,7 @@ void http_context::init(Socket *sock) {
     bind(sock);
 }
 
-void http_context::bind(Socket *sock) {
+void HttpContext::bind(Socket *sock) {
     private_data = sock;
     co_socket = 1;
     send = http_context_send_data;
@@ -301,7 +304,7 @@ void php_swoole_http_server_coro_minit(int module_number) {
                         nullptr,
                         "Co\\Http\\Server",
                         swoole_http_server_coro_methods);
-    SW_SET_CLASS_SERIALIZABLE(swoole_http_server_coro, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SW_SET_CLASS_NOT_SERIALIZABLE(swoole_http_server_coro);
     SW_SET_CLASS_CLONEABLE(swoole_http_server_coro, sw_zend_class_clone_deny);
     SW_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_http_server_coro, sw_zend_class_unset_property_deny);
     SW_SET_CLASS_CREATE_WITH_ITS_OWN_HANDLERS(swoole_http_server_coro);
@@ -435,7 +438,8 @@ static PHP_METHOD(swoole_http_server_coro, start) {
     zend_fcall_info_cache fci_cache;
     zval zcallback;
     ZVAL_STRING(&zcallback, "onAccept");
-    if (!sw_zend_is_callable_at_frame(&zcallback, ZEND_THIS, execute_data, 0, &func_name, nullptr, &fci_cache, nullptr)) {
+    if (!sw_zend_is_callable_at_frame(
+            &zcallback, ZEND_THIS, execute_data, 0, &func_name, nullptr, &fci_cache, nullptr)) {
         php_swoole_fatal_error(E_CORE_ERROR, "function '%s' is not callable", func_name);
         return;
     }
@@ -545,8 +549,8 @@ static PHP_METHOD(swoole_http_server_coro, onAccept) {
 
     Socket *sock = php_swoole_get_socket(zconn);
     sock->set_buffer_allocator(sw_zend_string_allocator());
-    swString *buffer = sock->get_read_buffer();
-    http_context *ctx = nullptr;
+    String *buffer = sock->get_read_buffer();
+    HttpContext *ctx = nullptr;
     bool header_completed = false;
     off_t header_crlf_offset = 0;
 
@@ -560,21 +564,22 @@ static PHP_METHOD(swoole_http_server_coro, onAccept) {
 #endif
 
     while (true) {
-        _recv_request: {
-            ssize_t retval = sock->recv(buffer->str + buffer->length, buffer->size - buffer->length);
-            if (sw_unlikely(retval <= 0)) {
-                break;
-            }
-            buffer->length += retval;
+    _recv_request : {
+        ssize_t retval = sock->recv(buffer->str + buffer->length, buffer->size - buffer->length);
+        if (sw_unlikely(retval <= 0)) {
+            break;
         }
+        buffer->length += retval;
+    }
 
-        _parse_request:
+    _parse_request:
         if (!ctx) {
             ctx = hs->create_context(sock, zconn);
         }
 
         if (!header_completed) {
-            if (swoole_strnpos(buffer->str + header_crlf_offset, buffer->length - header_crlf_offset, ZEND_STRL("\r\n\r\n")) < 0) {
+            if (swoole_strnpos(
+                    buffer->str + header_crlf_offset, buffer->length - header_crlf_offset, ZEND_STRL("\r\n\r\n")) < 0) {
                 if (buffer->length == buffer->size) {
                     ctx->response.status = SW_HTTP_REQUEST_ENTITY_TOO_LARGE;
                     break;
@@ -590,12 +595,12 @@ static PHP_METHOD(swoole_http_server_coro, onAccept) {
         size_t parsed_n = ctx->parse(buffer->str + buffer->offset, buffer->length - buffer->offset);
         buffer->offset += parsed_n;
 
-        swTraceLog(SW_TRACE_CO_HTTP_SERVER,
-                   "parsed_n=%ld, length=%ld, offset=%ld, completed=%d",
-                   parsed_n,
-                   buffer->length,
-                   buffer->offset,
-                   ctx->completed);
+        swoole_trace_log(SW_TRACE_CO_HTTP_SERVER,
+                         "parsed_n=%ld, length=%ld, offset=%ld, completed=%d",
+                         parsed_n,
+                         buffer->length,
+                         buffer->offset,
+                         ctx->completed);
 
         if (!ctx->completed) {
             if (ctx->parser.state == s_dead) {
@@ -689,7 +694,7 @@ static PHP_METHOD(swoole_http_server_coro, shutdown) {
 
 #ifdef SW_USE_HTTP2
 static void http2_server_onRequest(Http2Session *session, Http2Stream *stream) {
-    http_context *ctx = stream->ctx;
+    HttpContext *ctx = stream->ctx;
     http_server *hs = (http_server *) session->private_data;
     Socket *sock = (Socket *) ctx->private_data;
     zval *zserver = ctx->request.zserver;
