@@ -128,6 +128,7 @@ enum swErrorCode {
     SW_ERROR_WEBSOCKET_BAD_OPCODE,
     SW_ERROR_WEBSOCKET_UNCONNECTED,
     SW_ERROR_WEBSOCKET_HANDSHAKE_FAILED,
+    SW_ERROR_WEBSOCKET_PACK_FAILED,
 
     /**
      * server global error
@@ -143,11 +144,12 @@ enum swErrorCode {
     SW_ERROR_SERVER_SEND_IN_MASTER,
     SW_ERROR_SERVER_INVALID_REQUEST,
     SW_ERROR_SERVER_CONNECT_FAIL,
+    SW_ERROR_SERVER_INVALID_COMMAND,
 
     /**
      * Process exit timeout, forced to end.
      */
-    SW_ERROR_SERVER_WORKER_EXIT_TIMEOUT,
+    SW_ERROR_SERVER_WORKER_EXIT_TIMEOUT = 9101,
     SW_ERROR_SERVER_WORKER_ABNORMAL_PIPE_DATA,
     SW_ERROR_SERVER_WORKER_UNPROCESSED_DATA,
 
@@ -185,6 +187,6 @@ class Exception {
     int code;
     const char *msg;
 
-    Exception(int code);
+    Exception(int code) throw();
 };
-}
+}  // namespace swoole
