@@ -16,8 +16,12 @@
 
 #include "swoole.h"
 
+#include <unordered_set>
+
+static std::unordered_set<int> ignored_errors;
+
 namespace swoole {
-Exception::Exception(int code) : code(code) {
+Exception::Exception(int code) throw() : code(code) {
     msg = swoole_strerror(code);
 }
 }  // namespace swoole
@@ -163,6 +167,8 @@ const char *swoole_strerror(int code) {
         return "Websocket unconnected";
     case SW_ERROR_WEBSOCKET_HANDSHAKE_FAILED:
         return "Websocket handshake failed";
+    case SW_ERROR_WEBSOCKET_PACK_FAILED:
+        return "Websocket pack failed";
     case SW_ERROR_SERVER_MUST_CREATED_BEFORE_CLIENT:
         return "Server must created before client";
     case SW_ERROR_SERVER_TOO_MANY_SOCKET:
@@ -185,6 +191,8 @@ const char *swoole_strerror(int code) {
         return "Server invalid request";
     case SW_ERROR_SERVER_CONNECT_FAIL:
         return "Server connect fail";
+    case SW_ERROR_SERVER_INVALID_COMMAND:
+        return "Server invalid command";
     case SW_ERROR_SERVER_WORKER_EXIT_TIMEOUT:
         return "Server worker exit timeout";
     case SW_ERROR_SERVER_WORKER_ABNORMAL_PIPE_DATA:
@@ -242,3 +250,11 @@ const char *swoole_strerror(int code) {
 void swoole_throw_error(int code) {
     throw swoole::Exception(code);
 }
+
+void swoole_ignore_error(int code) {
+    ignored_errors.insert(code);
+}
+
+bool swoole_is_ignored_error(int code) {
+    return ignored_errors.find(code) != ignored_errors.end();
+}
