@@ -56,7 +56,7 @@ void thread_context_clean() {
     g_lock.unlock();
 }
 
-Context::Context(size_t stack_size, const coroutine_func_t &fn, void *private_data)
+Context::Context(size_t stack_size, const CoroutineFunc &fn, void *private_data)
     : fn_(fn), private_data_(private_data) {
     end_ = false;
     lock_.lock();
