@@ -311,6 +311,7 @@ static int Port_onRead_raw(Reactor *reactor, ListenPort *port, Event *event) {
     Socket *_socket = event->socket;
     Connection *conn = (Connection *) _socket->object;
     Server *serv = (Server *) reactor->ptr;
+    RecvData rdata{};
 
     String *buffer = serv->get_recv_buffer(_socket);
     if (!buffer) {
@@ -335,7 +336,9 @@ static int Port_onRead_raw(Reactor *reactor, ListenPort *port, Event *event) {
         return SW_OK;
     } else {
         buffer->offset = buffer->length = n;
-        return Server::dispatch_task(&port->protocol, _socket, buffer->str, n);
+        rdata.info.len = n;
+        rdata.data = buffer->str;
+        return Server::dispatch_task(&port->protocol, _socket, &rdata);
     }
 }
 
@@ -379,6 +382,7 @@ static int Port_onRead_http(Reactor *reactor, ListenPort *port, Event *event) {
     Socket *_socket = event->socket;
     Connection *conn = (Connection *) _socket->object;
     Server *serv = (Server *) reactor->ptr;
+    RecvData dispatch_data {};
 
     if (conn->websocket_status >= websocket::STATUS_HANDSHAKE) {
         if (conn->http_upgrade == 0) {
@@ -518,10 +522,10 @@ _parse:
     if (!request->header_parsed) {
         request->parse_header_info();
         swoole_trace_log(SW_TRACE_SERVER,
-                   "content-length=%u, keep-alive=%u, chunked=%u",
-                   request->content_length_,
-                   request->keep_alive,
-                   request->chunked);
+                         "content-length=%u, keep-alive=%u, chunked=%u",
+                         request->content_length_,
+                         request->keep_alive,
+                         request->chunked);
     }
 
     // content length (equal to 0) or (field not found but not chunked)
@@ -540,7 +544,9 @@ _parse:
             // send static file content directly in the reactor thread
             if (!serv->enable_static_handler || !serv->select_static_handler(request, conn)) {
                 // dynamic request, dispatch to worker
-                Server::dispatch_task(protocol, _socket, buffer->str, request->header_length_);
+                dispatch_data.info.len = request->header_length_;
+                dispatch_data.data = buffer->str;
+                Server::dispatch_task(protocol, _socket, &dispatch_data);
             }
             if (!conn->active || _socket->removed) {
                 return SW_OK;
@@ -614,11 +620,12 @@ _parse:
             if (request->has_expect_header()) {
                 _socket->send(SW_STRL(SW_HTTP_100_CONTINUE_PACKET), 0);
             } else {
-                swoole_trace_log(SW_TRACE_SERVER,
-                           "PostWait: request->content_length=%d, buffer->length=%zu, request->header_length=%d\n",
-                           request->content_length,
-                           buffer_->length,
-                           request->header_length);
+                swoole_trace_log(
+                    SW_TRACE_SERVER,
+                    "PostWait: request->content_length=%d, buffer->length=%zu, request->header_length=%d\n",
+                    request->content_length,
+                    buffer_->length,
+                    request->header_length);
             }
 #endif
             goto _recv_data;
@@ -636,7 +643,9 @@ _parse:
     }
 
     buffer->offset = request_length;
-    Server::dispatch_task(protocol, _socket, buffer->str, buffer->length);
+    dispatch_data.data = buffer->str;
+    dispatch_data.info.len = buffer->length;
+    Server::dispatch_task(protocol, _socket, &dispatch_data);
 
     if (conn->active && !_socket->removed) {
         serv->destroy_http_request(conn);
@@ -723,4 +732,34 @@ void ListenPort::close() {
     }
 }
 
+const char *ListenPort::get_protocols() {
+    if (is_dgram()) {
+        return "dgram";
+    }
+    if (open_eof_check) {
+        return "eof";
+    } else if (open_length_check) {
+        return "length";
+    } else if (open_http_protocol) {
+#ifdef SW_USE_HTTP2
+        if (open_http2_protocol && open_websocket_protocol) {
+            return "http|http2|websocket";
+        } else if (open_http2_protocol) {
+            return "http|http2";
+        } else
+#endif
+            if (open_websocket_protocol) {
+            return "http|websocket";
+        } else {
+            return "http";
+        }
+    } else if (open_mqtt_protocol) {
+        return "mqtt";
+    } else if (open_redis_protocol) {
+        return "redis";
+    } else {
+        return "raw";
+    }
+}
+
 }  // namespace swoole
