@@ -31,7 +31,7 @@ static int ReactorProcess_reuse_port(ListenPort *ls);
 
 static bool Server_is_single(Server *serv) {
     return serv->worker_num == 1 && serv->task_worker_num == 0 && serv->max_request == 0 &&
-           serv->user_worker_list == nullptr;
+           serv->user_worker_list.empty();
 }
 
 int Server::create_reactor_processes() {
@@ -91,6 +91,8 @@ int Server::start_reactor_processes() {
     gs->event_workers.use_msgqueue = 0;
     gs->event_workers.main_loop = ReactorProcess_loop;
     gs->event_workers.onWorkerNotFound = Server::wait_other_worker;
+    memcpy(workers, gs->event_workers.workers, sizeof(*workers) * worker_num);
+    gs->event_workers.workers = workers;
 
     SW_LOOP_N(worker_num) {
         gs->event_workers.workers[i].pool = &gs->event_workers;
@@ -98,6 +100,11 @@ int Server::start_reactor_processes() {
         gs->event_workers.workers[i].type = SW_PROCESS_WORKER;
     }
 
+    init_ipc_max_size();
+    if (create_pipe_buffers() < 0) {
+        return SW_ERR;
+    }
+
     // single worker
     if (Server_is_single(this)) {
         int retval = ReactorProcess_loop(&gs->event_workers, &gs->event_workers.workers[0]);
@@ -121,16 +128,14 @@ int Server::start_reactor_processes() {
         }
     }
 
-    /**
-     * create user worker process
-     */
-    if (user_worker_list) {
-        user_workers = (Worker *) sw_shm_calloc(user_worker_num, sizeof(Worker));
+    // create user worker process
+    if (!user_worker_list.empty()) {
+        user_workers = (Worker *) sw_shm_calloc(get_user_worker_num(), sizeof(Worker));
         if (user_workers == nullptr) {
             swoole_sys_warning("gmalloc[server->user_workers] failed");
             return SW_ERR;
         }
-        for (auto worker : *user_worker_list) {
+        for (auto worker : user_worker_list) {
             /**
              * store the pipe object
              */
@@ -152,15 +157,11 @@ int Server::start_reactor_processes() {
      */
     SwooleG.use_signalfd = 0;
 
+    gs->event_workers.onWorkerMessage = read_worker_message;
     gs->event_workers.start();
 
     init_signal_handler();
 
-    if (onStart) {
-        swoole_warning("The onStart event with SWOOLE_BASE is deprecated");
-        onStart(this);
-    }
-
     if (onManagerStart) {
         onManagerStart(this);
     }
@@ -182,82 +183,67 @@ int Server::start_reactor_processes() {
 }
 
 static int ReactorProcess_onPipeRead(Reactor *reactor, Event *event) {
-    EventData task;
     SendData _send;
     Server *serv = (Server *) reactor->ptr;
     Factory *factory = serv->factory;
-    String *output_buffer;
+    PipeBuffer *pipe_buffer = serv->message_bus.get_buffer();
 
-    ssize_t retval = read(event->fd, &task, sizeof(task));
+    ssize_t retval = serv->message_bus.read(event->socket);
     if (retval <= 0) {
-        return SW_ERR;
-    } else if ((size_t) retval != task.info.len + sizeof(_send.info)) {
-        swoole_warning("bad pipeline data");
         return SW_OK;
     }
 
-    switch (task.info.type) {
-    case SW_SERVER_EVENT_PIPE_MESSAGE:
-        serv->onPipeMessage(serv, &task);
+    switch (pipe_buffer->info.type) {
+    case SW_SERVER_EVENT_PIPE_MESSAGE: {
+        serv->onPipeMessage(serv, (EventData *) pipe_buffer);
         break;
-    case SW_SERVER_EVENT_FINISH:
-        serv->onFinish(serv, &task);
+    }
+    case SW_SERVER_EVENT_FINISH: {
+        serv->onFinish(serv, (EventData *) pipe_buffer);
         break;
-    case SW_SERVER_EVENT_SEND_FILE:
-        _send.info = task.info;
-        _send.data = task.data;
+    }
+    case SW_SERVER_EVENT_SEND_FILE: {
+        _send.info = pipe_buffer->info;
+        _send.data = pipe_buffer->data;
         factory->finish(&_send);
         break;
-    case SW_SERVER_EVENT_PROXY_START:
-    case SW_SERVER_EVENT_PROXY_END:
-        if (task.info.reactor_id < 0 || task.info.reactor_id >= (int16_t) serv->get_all_worker_num()) {
-            swoole_warning("invalid worker_id=%d", task.info.reactor_id);
+    }
+    case SW_SERVER_EVENT_SEND_DATA: {
+        if (pipe_buffer->info.reactor_id < 0 || pipe_buffer->info.reactor_id >= (int16_t) serv->get_all_worker_num()) {
+            swoole_warning("invalid worker_id=%d", pipe_buffer->info.reactor_id);
             return SW_OK;
         }
-        output_buffer = SwooleWG.output_buffer[task.info.reactor_id];
-        output_buffer->append(task.data, task.info.len);
-        if (task.info.type == SW_SERVER_EVENT_PROXY_END) {
-            memcpy(&_send.info, &task.info, sizeof(_send.info));
-            _send.info.type = SW_SERVER_EVENT_RECV_DATA;
-            _send.data = output_buffer->str;
-            _send.info.len = output_buffer->length;
-            factory->finish(&_send);
-            output_buffer->clear();
-        }
+        auto packet = serv->message_bus.get_packet();
+        memcpy(&_send.info, &pipe_buffer->info, sizeof(_send.info));
+        _send.info.type = SW_SERVER_EVENT_RECV_DATA;
+        _send.data = packet.data;
+        _send.info.len = packet.length;
+        factory->finish(&_send);
         break;
-    case SW_SERVER_EVENT_CLOSE:
-        factory->end(task.info.fd, Server::CLOSE_ACTIVELY);
+    }
+    case SW_SERVER_EVENT_CLOSE: {
+        factory->end(pipe_buffer->info.fd, Server::CLOSE_ACTIVELY);
         break;
+    }
+    case SW_SERVER_EVENT_COMMAND_REQUEST: {
+        serv->call_command_handler(serv->message_bus, SwooleWG.worker->id, serv->get_worker(0)->pipe_master);
+        break;
+    }
+    case SW_SERVER_EVENT_COMMAND_RESPONSE: {
+        int64_t request_id = pipe_buffer->info.fd;
+        auto packet = serv->message_bus.get_packet();
+        serv->call_command_callback(request_id, std::string(packet.data, packet.length));
+        break;
+    }
     default:
         break;
     }
+
+    serv->message_bus.pop();
+
     return SW_OK;
 }
 
-static int ReactorProcess_alloc_output_buffer(size_t n_buffer) {
-    SwooleWG.output_buffer = (String **) sw_malloc(sizeof(String *) * n_buffer);
-    if (SwooleWG.output_buffer == nullptr) {
-        swoole_error("malloc for SwooleWG.output_buffer failed");
-        return SW_ERR;
-    }
-
-    SW_LOOP_N(n_buffer) {
-        SwooleWG.output_buffer[i] = new String(SW_BUFFER_SIZE_BIG);
-        if (SwooleWG.output_buffer[i] == nullptr) {
-            swoole_error("output_buffer init failed");
-            return SW_ERR;
-        }
-    }
-    return SW_OK;
-}
-
-static void ReactorProcess_free_output_buffer(size_t n_buffer) {
-    SW_LOOP_N(n_buffer) {
-        delete SwooleWG.output_buffer[i];
-    }
-    sw_free(SwooleWG.output_buffer);
-}
-
 static int ReactorProcess_loop(ProcessPool *pool, Worker *worker) {
     Server *serv = (Server *) pool->ptr;
 
@@ -270,15 +256,10 @@ static int ReactorProcess_loop(ProcessPool *pool, Worker *worker) {
     }
     SwooleWG.max_request = serv->max_request;
     SwooleWG.worker = worker;
-
     SwooleTG.id = 0;
-    if (worker->id == 0) {
-        SwooleTG.update_time = 1;
-    }
 
     serv->init_worker(worker);
 
-    // create reactor
     if (!SwooleTG.reactor) {
         if (swoole_event_init(0) < 0) {
             return SW_ERR;
@@ -291,16 +272,10 @@ static int ReactorProcess_loop(ProcessPool *pool, Worker *worker) {
         SwooleTG.timer->reinit(reactor);
     }
 
-    size_t n_buffer = serv->get_all_worker_num();
-    if (ReactorProcess_alloc_output_buffer(n_buffer)) {
-        return SW_ERR;
-    }
-
     for (auto ls : serv->ports) {
 #ifdef HAVE_REUSEPORT
         if (ls->is_stream() && serv->enable_reuse_port) {
             if (ReactorProcess_reuse_port(ls) < 0) {
-                ReactorProcess_free_output_buffer(n_buffer);
                 swoole_event_free();
                 return SW_ERR;
             }
@@ -348,7 +323,7 @@ static int ReactorProcess_loop(ProcessPool *pool, Worker *worker) {
     // task workers
     if (serv->task_worker_num > 0) {
         if (serv->task_ipc_mode == Server::TASK_IPC_UNIXSOCK) {
-            for (uint32_t i = 0; i < serv->gs->task_workers.worker_num; i++) {
+            SW_LOOP_N(serv->gs->task_workers.worker_num) {
                 serv->gs->task_workers.workers[i].pipe_master->set_nonblock();
             }
         }
@@ -356,8 +331,8 @@ static int ReactorProcess_loop(ProcessPool *pool, Worker *worker) {
 
     serv->init_reactor(reactor);
 
-    // single server trigger onStart event
-    if (Server_is_single(serv)) {
+    if (worker->id == 0) {
+        SwooleTG.update_time = 1;
         if (serv->onStart) {
             serv->onStart(serv);
         }
@@ -368,7 +343,6 @@ static int ReactorProcess_loop(ProcessPool *pool, Worker *worker) {
      */
     if ((serv->master_timer = swoole_timer_add(1000, true, Server::timer_callback, serv)) == nullptr) {
     _fail:
-        ReactorProcess_free_output_buffer(n_buffer);
         swoole_event_free();
         return SW_ERR;
     }
@@ -405,7 +379,6 @@ static int ReactorProcess_loop(ProcessPool *pool, Worker *worker) {
 
     swoole_event_free();
     serv->worker_stop_callback();
-    ReactorProcess_free_output_buffer(n_buffer);
 
     return retval;
 }
