@@ -43,7 +43,7 @@ TimerNode *swoole_timer_add(long ms, bool persistent, const TimerCallback &callb
 
 bool swoole_timer_del(TimerNode *tnode) {
     if (!swoole_timer_is_available()) {
-        swWarn("timer is not available");
+        swoole_warning("timer is not available");
         return false;
     }
     return SwooleTG.timer->remove(tnode);
@@ -51,7 +51,7 @@ bool swoole_timer_del(TimerNode *tnode) {
 
 void swoole_timer_delay(TimerNode *tnode, long delay_ms) {
     if (!swoole_timer_is_available()) {
-        swWarn("timer is not available");
+        swoole_warning("timer is not available");
         return;
     }
     return SwooleTG.timer->delay(tnode, delay_ms);
@@ -59,7 +59,7 @@ void swoole_timer_delay(TimerNode *tnode, long delay_ms) {
 
 long swoole_timer_after(long ms, const TimerCallback &callback, void *private_data) {
     if (ms <= 0) {
-        swWarn("Timer must be greater than 0");
+        swoole_warning("Timer must be greater than 0");
         return SW_ERR;
     }
     TimerNode *tnode = swoole_timer_add(ms, false, callback, private_data);
@@ -72,7 +72,7 @@ long swoole_timer_after(long ms, const TimerCallback &callback, void *private_da
 
 long swoole_timer_tick(long ms, const TimerCallback &callback, void *private_data) {
     if (ms <= 0) {
-        swWarn("Timer must be greater than 0");
+        swoole_warning("Timer must be greater than 0");
         return SW_ERR;
     }
     TimerNode *tnode = swoole_timer_add(ms, true, callback, private_data);
@@ -85,7 +85,7 @@ long swoole_timer_tick(long ms, const TimerCallback &callback, void *private_dat
 
 bool swoole_timer_exists(long timer_id) {
     if (!swoole_timer_is_available()) {
-        swWarn("timer is not available");
+        swoole_warning("timer is not available");
         return false;
     }
     TimerNode *tnode = SwooleTG.timer->get(timer_id);
@@ -94,7 +94,7 @@ bool swoole_timer_exists(long timer_id) {
 
 bool swoole_timer_clear(long timer_id) {
     if (!swoole_timer_is_available()) {
-        swWarn("timer is not available");
+        swoole_warning("timer is not available");
         return false;
     }
     return SwooleTG.timer->remove(SwooleTG.timer->get(timer_id));
@@ -102,7 +102,7 @@ bool swoole_timer_clear(long timer_id) {
 
 TimerNode *swoole_timer_get(long timer_id) {
     if (!swoole_timer_is_available()) {
-        swWarn("timer is not available");
+        swoole_warning("timer is not available");
         return nullptr;
     }
     return SwooleTG.timer->get(timer_id);
@@ -110,7 +110,7 @@ TimerNode *swoole_timer_get(long timer_id) {
 
 void swoole_timer_free() {
     if (!swoole_timer_is_available()) {
-        swWarn("timer is not available");
+        swoole_warning("timer is not available");
         return;
     }
     delete SwooleTG.timer;
@@ -120,7 +120,7 @@ void swoole_timer_free() {
 
 int swoole_timer_select() {
     if (!swoole_timer_is_available()) {
-        swWarn("timer is not available");
+        swoole_warning("timer is not available");
         return SW_ERR;
     }
     return SwooleTG.timer->select();
