@@ -4,17 +4,26 @@
  *
  * @link     https://www.swoole.com
  * @contact  team@swoole.com
- * @license  https://github.com/swoole/library/blob/master/LICENSE
+ * @license  https://github.com/swoole/swoole-src/blob/master/LICENSE
  */
 
 define('ROOT_DIR', dirname(__DIR__));
 define('LIBRARY_DIR', ROOT_DIR . '/library');
 define('LIBRARY_SRC_DIR', LIBRARY_DIR . '/src');
 
-define('EMOJI_OK', '✅');
-define('EMOJI_SUCCESS', '🚀');
-define('EMOJI_ERROR', '❌');
-define('EMOJI_WARN', '⚠️');
+if (!defined('EMOJI_OK')) {
+    define('EMOJI_OK', '✅');
+}
+if (!defined('EMOJI_SUCCESS')) {
+    define('EMOJI_SUCCESS', '🚀');
+}
+if (!defined('EMOJI_ERROR')) {
+    define('EMOJI_ERROR', '❌');
+}
+if (!defined('EMOJI_WARN')) {
+    define('EMOJI_WARN', '⚠️');
+}
+
 define('SWOOLE_SOURCE_ROOT', dirname(__DIR__) . '/');
 
 if (!defined('SWOOLE_COLOR_RED')) {
@@ -123,6 +132,20 @@ function scan_dir(string $dir, callable $filter = null): array
     return array_values($filter ? array_filter($files, $filter) : $files);
 }
 
+function scan_dir_recursive(string $dir, callable $filter = null): array
+{
+    $result = [];
+    $files = scan_dir($dir, $filter);
+    foreach ($files as $f) {
+        if (is_dir($f)) {
+            $result = array_merge($result, scan_dir_recursive($f, $filter));
+        } else {
+            $result[] = $f;
+        }
+    }
+    return $result;
+}
+
 function file_size(string $filename, int $decimals = 2): string
 {
     $bytes = filesize($filename);
@@ -163,3 +186,46 @@ function swoole_source_list(array $ext_list = [], array $excepts = []): array
 
     return $source_list;
 }
+
+function swoole_library_files()
+{
+    $files = [];
+
+    $file_spl_objects = new \RecursiveIteratorIterator(
+        new \RecursiveDirectoryIterator(LIBRARY_SRC_DIR, \RecursiveDirectoryIterator::SKIP_DOTS),
+        \RecursiveIteratorIterator::LEAVES_ONLY
+    );
+
+    foreach ($file_spl_objects as $full_file_name => $file_spl_object) {
+        $files[] = str_replace(LIBRARY_SRC_DIR . '/', '', $full_file_name);
+    }
+
+    return $files;
+}
+
+function swoole_remove_php_comments($code)
+{
+    $newCode = '';
+    $commentTokens = array(T_COMMENT);
+
+    if (defined('T_DOC_COMMENT')) {
+        $commentTokens[] = T_DOC_COMMENT;
+    }
+
+    if (defined('T_ML_COMMENT')) {
+        $commentTokens[] = T_ML_COMMENT;
+    }
+
+    $tokens = token_get_all($code);
+    foreach ($tokens as $token) {
+        if (is_array($token)) {
+            if (in_array($token[0], $commentTokens)) {
+                continue;
+            }
+            $token = $token[1];
+        }
+        $newCode .= $token;
+    }
+
+    return $newCode;
+}
